/*
 * A parallel reduction kernel over a local array [A]
 */

#define tid get_local_id(0)
#define N   get_local_size(0)

#define ispow2(x) ((((x-1)&x) == 0) & (x != 0))

__kernel void reduction(__local int *A) {
  __requires(ispow2(N));

  for (uint offset=N/2; offset > 0; offset >>= 1) {
    __assert(ispow2(offset));
    if (tid < offset) {
      A[tid] += A[tid+offset];
    }
    barrier(CLK_LOCAL_MEM_FENCE);
  }
}
