/*
 * Introducing assertions
 */

__kernel void simple(__local int *A) {

  uint tid = get_local_id(0);
  A[tid] = tid;

  __assert(A[tid] == tid);
  __assert(A[tid] != get_local_size(0));
  __assert(__implies(
    __write(A),
    __write_offset(A)/sizeof(int) == tid));

  // write your own assertions of the following form
  //   !e                                           logical not
  __assert(!false);

  //   e1 && e2                                     logical and
  int x = 42;
  int y = 24;
  __assert(x == 42 && y == 24);

  //   e1 || e2                                     logical or
  __assert(x == 42 || y == 0);

  //   __implies(e1, e2)                            implication
  int z = 0;
  if (x == 42) z = 1;
  __assert(__implies(x == 42, z == 1));
  __assert(__implies(x ==  0, z == 99));

  // give an example of an assertion that will always fail
  __assert(false);

  // what do you think about
  __assert(__implies(tid == -1, A[tid] == tid+1));
  // ?

  // The predicate is true.
  // Truth table of implication __implies(e1,e2) is equivalent to !e1 || e2
  //   e1 e2 __implies(e1,e2)
  //   F  F  T
  //   F  T  T
  //   T  F  F
  //   T  T  T
  // Think of implication as 'truth-preserving'
}
