/*
 * This kernel strides along the array A in group-sized chunks.
 */

// First try verifying with
//     gpuverify --local_size=1024 --num_groups=1            stride.cl
// Now let's turn off GPUVerify's automatic loop inference
//     gpuverify --local_size=1024 --num_groups=1 --no-infer stride.cl

#define N 8

__kernel void stride(__local int *A, __local int *B) {
  uint tid = get_local_id(0);

  // Q1: can you find a set of loop invariants that allow GPUVerify to pass when
  // using the "--no-infer" flag?
  // Hint: you will need an invariant to bound the variable [i]
  // and also an invariant of the form:
  //   __implies(__write(A), __write_offset(A)/sizeof(int) /* ???*/)
  for (int i=0; 
       __invariant(i <= N), 
       __invariant(__write_implies(A, __write_offset(A)/sizeof(int) % get_local_size(0) == tid)), 
       i<N; i++) {
    A[i*get_local_size(0)+tid] = tid;
  }

  // Q2: Does this loop have the same effect on array [B]?
  // Try GPUVerify on this loop too.
  // Don't worry about finding invariants for this loop; just use GPUVerify's
  // built-in inference engine.
#if 1
  for (int i=get_local_id(0);
      // Here are the invariants necessary if you turn invariant inference off
      // using --no-infer
      // There is a subtle difference between invariant and global_invariant,
      // which we won't go into here.
      __global_invariant(i < (N+1)*get_local_size(0)),
      __global_invariant(i % get_local_size(0) == get_local_id(0)),
      __global_invariant(__write_implies(B, __write_offset(B)/sizeof(int) % get_local_size(0) == get_local_id(0))),
      i<N*get_local_size(0); i+=get_local_size(0)) {
    B[i] = tid;
  }
#endif

#ifdef FORCE_FAIL
  __assert(false);
#endif
}
