/*
 * This kernel strides along the array A in group-sized chunks.
 */

// First try verifying with
//     gpuverify --local_size=1024 --num_groups=1            stride.cl
// Now let's turn off GPUVerify's automatic loop inference
//     gpuverify --local_size=1024 --num_groups=1 --no-infer stride.cl

#define N 8

__kernel void stride(__local int *A) {
  uint tid = get_local_id(0);

  // Q1: can you find a set of loop invariants that allow GPUVerify to pass when
  // using the "--no-infer" flag?
  // Hint: you will need an invariant to bound the variable [i]
  // and also an invariant of the form:
  //   __implies(__write(A), __write_offset(A)/sizeof(int) /* ???*/)
  for (int i=0;
        i<N; i++) {
    A[i*get_local_size(0)+tid] = tid;
  }

  // Q2: Does this loop have the same effect on array [B]?
  // Try GPUVerify on this loop too.
  // Don't worry about finding invariants for this loop; just use GPUVerify's
  // built-in inference engine.
#if 0
  for (int i=get_local_id(0); i<N*get_local_size(0); i+=get_local_size(0)) {
    A[i] = tid;
  }
#endif
}
