/*
 * A kernel that defeats the shared-state abstraction used inside GPUVerify.
 */

#define tid get_local_id(0)
#define N   get_local_size(0)

__kernel void dbl_indirect(__local int *A) {
  A[tid] = tid;
  // However, with an experimental feature of GPUVerify, it is possible to
  // verify this kernel using a "Barrier Invariant": a property that is
  // true for shared-state at a barrier and can be preserved across
  // the shared-state abstraction
  __barrier_invariant(A[tid] == tid, tid, (tid+1)%N);
  barrier(CLK_LOCAL_MEM_FENCE);
  A[A[(tid+1)%N]] = tid;
}
