/*
 * A kernel that defeats the shared-state abstraction used inside GPUVerify.
 * See also answer/dbl_indirect.cl for a new experimental feature that allows
 * us to bypass this restriction.
 */

#define tid get_local_id(0)
#define N   get_local_size(0)

__kernel void dbl_indirect(__local int *A) {
  A[tid] = tid;
  barrier(CLK_LOCAL_MEM_FENCE);
  // Due to shared-state abstraction, the array [A] is havoced (set
  // non-deterministically) after the barrier. GPUVerify must do this
  // to overapproximate the threads that were not modelled by its 2-thread
  // reduction.
  A[A[(tid+1)%N]] = tid; // hence, GPUVerify reports a race here
}
