/*
 * Iterate over the edges of a graph in parallel. This kernel relies on an
 * outer property of the [edgecolour] array.
 */

// Try verifying this with
//    gpuverify --local_size=1024 --num_groups=1 graph.cl

#define MAX_COLOUR 8
#define tid get_local_id(0)
#define other_tid __other_int(tid)

__kernel void iterall_edges(__local uint2 *edges,
                            __local uint  *edgecolour,
                            __local float *node_val) {
  // Q1: write a precondition about the expected values inside [edgecolour]
  // that will enable GPUVerify to prove race-freedom of this kernel.
  __requires(true);
  // Hint: You need to write a *pairwise* property that involves 'tid'
  // and 'other_tid', given as macros above.

  uint colour = edgecolour[tid];
  uint n1 = edges[tid].lo;
  uint n2 = edges[tid].hi;
  for (uint c = 0; __invariant(__no_write(node_val)), c < MAX_COLOUR; c++) {
    if (c == colour) {
      float v, w; /* some arbitrary float value */
      node_val[n1] = v;
      node_val[n1] = w;
    }
    barrier(CLK_LOCAL_MEM_FENCE);
  }

}
